"use strict";

function _typeof(obj) { if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

var debug = require('debug')('seneca:entity');

var util = require('util');

var _ = require('lodash');

var common = require('./common');

var noop = common.noop;

var Entity =
/*#__PURE__*/
function () {
  function Entity(canon, seneca) {
    _classCallCheck(this, Entity);

    Object.defineProperty(this, '_seneca$', {
      value: seneca,
      enumerable: false
    });
    this._canon$ = canon;
    this.entity$ = this.canon$();
  }
  /**
   * @deprecated
   */


  _createClass(Entity, [{
    key: "log$",
    value: function log$() {
      this._seneca$.log.apply(this, arguments);
    }
  }, {
    key: "make$",

    /**
     * Create a blank entity.
     *
     * @example
     *   make$('foo/bar')
     *   make$({ name: 'foobar' })
     *   make$('foo/bar', { name: 'foobar' })
     *
     * @
     */
    value: function make$(arg1, arg2) {
      var assign = null,
          canon;

      if (typeof arg1 === 'string') {
        var inputCanon = Entity.parseCanon(arg1);
        canon = {
          zone: inputCanon.zone == null ? this._canon$.zone : inputCanon.zone,
          base: inputCanon.base == null ? this._canon$.base : inputCanon.base,
          name: inputCanon.name == null ? this._canon$.name : inputCanon.name
        };

        if (arg2 != null) {
          assign = arg2;
        }
      } else {
        assign = arg1;
        canon = this._canon$;
      }

      var entity = new Entity(canon, this._seneca$);
      debug('make (%o)', entity._canon$);

      if (assign != null) {
        for (var key in assign) {
          if (key[key.length - 1] !== '$') {
            entity[key] = assign[key];
          }
        }

        if (assign.id$ != null) {
          entity.id$ = assign.id$;
        }
      }

      return entity;
    } // save one

  }, {
    key: "save$",
    value: function save$(props, cb) {
      debug('save (%o)', this._canon$);
      var si = this._seneca$;

      if (_.isFunction(props)) {
        cb = props;
      } else if (_.isObject(props)) {
        this.data$(props);
      }

      si.act(this._entargs$({
        cmd: 'save'
      }), cb);
      return this;
    }
    /**
     * Provide native database driver
     *
     * @deprecated
     */

  }, {
    key: "native$",
    value: function native$(cb) {
      debug('native (%o)', this._canon$);

      this._seneca.act(this._entargs$({
        cmd: 'native'
      }), cb || noop);

      return this;
    }
    /**
     * load one. not a scalar query.
     */

  }, {
    key: "load$",
    value: function load$(qin, cb) {
      var query = this._resolveIdQuery$(qin, this);

      debug('load (%o): %o', this._canon$, query); // empty query gives empty result

      if (null == query) {
        return cb();
      }

      var args = {
        qent: this,
        q: query,
        cmd: 'load'
      };

      this._seneca$.act(this._entargs$(args), cb);

      return this;
    }
    /**
     * list zero or more
     * qin is optional, if omitted, list all.
     */

  }, {
    key: "list$",
    value: function list$(qin, cb) {
      var si = this._seneca$;
      var qent = this;
      var q = qin;

      if (_.isFunction(qin)) {
        q = {};
        cb = qin;
      }

      debug('list (%o): %o', this._canon$, q);
      si.act(this._entargs$({
        qent: qent,
        q: q,
        cmd: 'list'
      }), cb || noop);
      return this;
    }
    /**
     * Delete.
     */

  }, {
    key: "remove$",
    value: function remove$(qin, cb) {
      var self = this;
      var si = this._seneca$;

      var q = this._resolveIdQuery$(qin, this);

      cb = (_.isFunction(qin) ? qin : cb) || noop; // empty query means take no action

      if (null == q) {
        if (typeof qin === 'function') qin = cb();
        return cb();
      }

      debug('remove (%o): %o', this._canon$, q);
      si.act(this._entargs$({
        qent: self,
        q: q,
        cmd: 'remove'
      }), cb || noop);
      return this;
    }
  }, {
    key: "delete$",
    value: function delete$() {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      this.remove$.apply(this, args);
    }
  }, {
    key: "fields$",
    value: function fields$() {
      debug('fields');
      var fields = [];

      for (var key in this) {
        if (key[key.length - 1] !== '$') {
          fields.push(key);
        }
      }

      return fields;
    }
  }, {
    key: "is$",
    value: function is$(canonspec) {
      debug('is');
      var canon = canonspec ? canonspec.entity$ ? canonspec.canon$({
        object: true
      }) : Entity.parseCanon(canonspec) : null;
      if (!canon) return false;
      return util.inspect(this.canon$({
        object: true
      })) == util.inspect(canon);
    }
  }, {
    key: "canon$",
    value: function canon$(options) {
      debug('canon (%o): %o', this._canon$, options);
      var canon = this._canon$;

      if (!options || options.string) {
        return (canon.zone || '-') + '/' + (canon.base || '-') + '/' + (canon.name || '-');
      } else if (options.string$) {
        return '$' + (canon.zone || '-') + '/' + (canon.base || '-') + '/' + (canon.name || '-');
      } else if (options.object) {
        return {
          zone: canon.zone,
          base: canon.base,
          name: canon.name
        };
      } else if (options.object$) {
        return {
          zone$: canon.zone,
          base$: canon.base,
          name$: canon.name
        };
      } else {
        return [canon.zone, canon.base, canon.name];
      }
    }
  }, {
    key: "data$",
    value: function data$(data) {
      debug('data');

      if (data) {
        for (var key in data) {
          if (key[key.length - 1] !== '$') {
            this[key] = data[key];
          }
        }

        return this;
      } else {
        var result = {
          entity$: this.entity$
        };

        for (var _key2 in this) {
          if (_key2[_key2.length - 1] !== '$') {
            result[_key2] = this[_key2];
          }
        }

        return result;
      }
    }
  }, {
    key: "clone$",
    value: function clone$() {
      debug('clone');
      return this.make$(this.data$());
    }
    /**
     * @private
     */

  }, {
    key: "_entargs$",
    value: function _entargs$(args) {
      args.role = 'entity';
      args.ent = this;

      if (this._canon$.name != null) {
        args.name = this._canon$.name;
      }

      if (this._canon$.base != null) {
        args.base = this._canon$.base;
      }

      if (this._canon$.zone != null) {
        args.zone = this._canon$.zone;
      }

      return args;
    }
    /**
     * @private
     */

  }, {
    key: "_resolveIdQuery$",
    value: function _resolveIdQuery$(query, ent) {
      switch (_typeof(query)) {
        case 'number':
        case 'string':
          return {
            id: query
          };

        case 'function':
          return ent.id != null ? {
            id: ent.id
          } : query;

        case 'undefined':
          return ent.id != null ? {
            id: ent.id
          } : query;

        case 'object':
          if (query === null && ent.id != null) return {
            id: ent.id
          };
          return query;

        default:
          return query;
      }
    }
  }], [{
    key: "create",
    value: function create(input, assign, seneca) {
      var canon = typeof input === 'string' ? Entity.parseCanon(input) : input;
      var entity = new Entity(canon, seneca);
      debug('seneca.make (%o):', entity._canon$);

      if (assign) {
        for (var key in assign) {
          if (key[key.length - 1] !== '$') {
            entity[key] = assign[key];
          }
        }

        if (assign.id$ != null) {
          entity.id$ = assign.id$;
        }
      }

      return entity;
    }
  }, {
    key: "parseCanon",
    value: function parseCanon(str) {
      debug('parseCanon: %s', str);

      if (_.isArray(str)) {
        return {
          zone: str[0],
          base: str[1],
          name: str[2]
        };
      }

      if (_.isObject(str) && !_.isFunction(str)) return str;
      if (!_.isString(str)) return {};
      var match = /\$?((\w+|-)\/)?((\w+|-)\/)?(\w+|-)/.exec(str);

      if (match) {
        var zi = void 0 == match[4] ? 4 : 2,
            bi = void 0 == match[4] ? 2 : 4;
        var out = {};
        out.zone = '-' == match[zi] ? void 0 : match[zi];
        out.base = '-' == match[bi] ? void 0 : match[bi];
        out.name = '-' == match[5] ? void 0 : match[5];
        return out;
      }

      throw new Error(util.format('Invalid entity canon: %s; expected format zone/base/name'));
    }
  }]);

  return Entity;
}();

module.exports = Entity;